/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins.watermelon;

import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.ucam.srcf.assassins.AssassinStateViewer;
import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.PlayerViewer;
import org.ucam.srcf.assassins.PluginStateChange;
import org.ucam.srcf.assassins.TextQuestion;
import org.ucam.srcf.assassins.EventViewer;
import org.ucam.srcf.assassins.Plugin;
import org.ucam.srcf.assassins.StateChange;
import org.ucam.srcf.assassins.YesNoQuestion;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

/**
 * Interface for java classes which act as plug-in game modifiers.
 */
public class Watermelon extends Plugin
{
   public static String getDescription() { return "Set a `wanted' flag on assassins."; };
   /**
    * Called when the game is first created, to allow the plugin to do any
    * initialisation necessary.
    */
   protected void gameCreated() {}
   
   /**
    * Called whenever a state change is added to a canonical event.
    * @param sc The state change which has been added.
    * @param qs If questions have been asked, this will contain the replies.
    * @see Plugin.getQuestions
    */
   protected void stateChangeAdded(StateChange sc, Vector qs) {}
   
   /**
    * Called whenever an existing state change is about to be removed.
    * @param sc The state change which is about to be deleted.
    * @param qs If questions have been asked, this will contain the replies
    * @see Plugin.getQuestions
    */
   protected void stateChangeRemoved(StateChange sc, Vector qs) {}

   /**
    * Called whenever the event to which an existing state change is
    * attached has its time changed.
    * @param sc The state change affected.
    * @param oldtime The previous timing of the state change's associated
    *    event.
    * @param newtime The new timing of the event. Note that this can also
    *    be retrieved with {@link StateChange#getEvent} and
    *    {@link Event#getTime}, but is provided here for convenience.
    */
   protected void stateChangeTimeChanged(StateChange sc, int oldtime, int newtime)  throws AssassinSQLException
   {
   }
   
   /**
    * Called whenever the plugin is required to update or sanity-check the
    * current gamestate.
    */
   protected Vector updateState(Map state, Vector qs, EventViewer e) throws AssassinSQLException
   {
      if (Debug.debug) Debug.print(Watermelon.class, "updateState("+state+","+qs+")");
      Vector v = new Vector();
      Iterator i = state.keySet().iterator();
      int j = 0;
      while(i.hasNext()) {
         AssassinViewer a = (AssassinViewer) i.next();
         PlayerViewer p = a.getPlayerViewer();
         AssassinStateViewer asv = (AssassinStateViewer) state.get(a);
         boolean wanted = asv.getBoolProperty(this, "wanted");
         if (Debug.debug) Debug.print(Watermelon.class,"Wanted?: " + asv.getProperty(this, "wanted"));
         if (wanted) {
            if (0 != qs.size()) {
              YesNoQuestion q = (YesNoQuestion) qs.get(j++);
              if (Debug.debug) Debug.print(Watermelon.class, "Got answer: " + q.getAnswer());
              if (q.getAnswer()) {
                 vanilla.addStateChange(e, new PluginStateChange(a, this, "wanted", "false"));
              }
            }
            else {
               v.add(new YesNoQuestion("Redeem "+p.getName()+"?", false));
            }
         }
      }
      if (0 == qs.size()) return null;
      return v;
   }
   
   /**
    * Called by the filtering mechanism when the plugin needs to determine
    * whether an assassin has a given property such as &quot;police&quot;. If
    * the plugin does not recognise the named property, it should return
    * <code>false</code>.
    * @param a A viewer for the assassin being tested.
    * @param prop The name of the property being asked about.
    * @return True if the assassin has the property, false if not.
    */
   public  boolean assassinHasProperty(AssassinStateViewer asv, String prop, String value)
   {
      if (Debug.debug) Debug.print(Watermelon.class, "assassinHasProperty("+asv+","+prop+","+value+")");
      if (!"wanted".equals(prop)) return false;
      String wanted = asv.getProperty(this, "wanted");
      if (null == wanted) return "false".equals(value);
      if (wanted.equals(value)) return true;
      return false;
   }

   /**
    * Called during the updating of the gamestate to ask whether a given player
    * requires email to be sent to them during the current round of emails.
    * @param p A viewer for the player being asked about.
    * @return True if the plugin requires email to be sent to the player in
    *    question, false if not.
    */
   protected boolean playerNeedsEmail(AssassinStateViewer asv) 
   {
      return false;
   }

   /**
    * Called when the system has decided that email needs to be sent to a given
    * player. The plugin may return a string to be included in the email, or
    * null if it does not wish to include anything.
    * @param p A viewer for the player being emailed.
    * @return A string to be included in the email sent to the player, or null.
    *    The string will appear in its own paragraph when inserted into the
    *    email and so should not begin or end with a newline.
    */
   protected String generateEmail(AssassinStateViewer asv) 
   {
      return "";
   }

   /**
    * Called when an event is being added to get any queries the plugin
    * has for the user. The plugin should check all the event details and if it
    * needs more information it should ask relevent questions. The 
    * returnQuestions function is passed the same Vector with the answers 
    * filled out.
    * @param e The Event which is being added
    * @return A Vector of Questions
    * @see returnQuestions
    */
   protected Vector getQuestions(EventViewer e, Map state) throws AssassinSQLException
   {
      if (Debug.debug) Debug.print(Watermelon.class, "getQuestions("+e+","+state+")");
      Vector v = new Vector();
      Iterator i = e.getParticipants().values().iterator();
      while(i.hasNext()) {
         AssassinViewer a = (AssassinViewer) i.next();
         PlayerViewer p = a.getPlayerViewer();
         v.add(new YesNoQuestion("Make " + p.getName() + " wanted?", false));
         v.add(new TextQuestion("Crime:", ""));
      }
      return v;

    }
   
   /**
    * Returns the Questions with the answers filled out
    *
    * @param e The Event being added
    * @param qs The Questions which were asked
    */
   protected void returnQuestions(EventViewer e, Map state, Vector qs)  throws AssassinSQLException
   {
      if (Debug.debug) Debug.print(Watermelon.class, "returnQuestions("+e+","+state+","+qs+")");
      Map participants = e.getParticipants();
      Iterator i = participants.values().iterator();
      int j = 0;
      while (i.hasNext()) {         
         boolean wanted = ((YesNoQuestion) qs.get(j)).getAnswer();
         String crime = ((TextQuestion) qs.get(j+1)).getAnswer();
         AssassinViewer a = (AssassinViewer) i.next();
//         AssassinStateViewer as = (AssassinStateViewer) state.get(a);         
         if (wanted) {
            vanilla.addStateChange(e, new PluginStateChange(a, this, "wanted", "true"));
            vanilla.addStateChange(e, new PluginStateChange(a, this, "crime", crime));
         }
         j+=2;
      }
   }

}
