package org.ucam.srcf.assassins.xml;

// JAXP
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.ucam.srcf.assassins.exceptions.XMLParseException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Program to allow the autoumpire to have behaviour specified in an XML file.
 * Use JAXP to read in and create a DOM tree. Then as long as a reference is
 * kept to this object, we
 * can perform lookups on it for any required attribute.
 *
 * @author Mike Cripps <mac57@srcf.ucam.org>
 */

public class XMLParser {

   private String filename;
   private boolean validation;
   boolean ignoreWhitespace;
   boolean ignoreComments;
   boolean putCDATAIntoText;
   boolean createEntityRefs;
   Document doc;

   final String outputEncoding = "UTF-8";

   public XMLParser () throws XMLParseException {
      this ("config.xml");
   }

   public XMLParser(String filename) throws XMLParseException {
      this(filename,false,false,false,false,false);
   }

   public XMLParser(String f, boolean v, boolean iw, boolean ic, boolean cd, boolean e) throws XMLParseException {
      filename=f;
      validation=v;
      ignoreWhitespace=iw;
      ignoreComments=ic;
      putCDATAIntoText=cd;
      createEntityRefs=e;
      parse();
   }

   public void parse() throws XMLParseException {
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      dbf.setNamespaceAware(true);
      dbf.setValidating(validation);
      dbf.setIgnoringComments(ignoreComments);
      dbf.setIgnoringElementContentWhitespace(ignoreWhitespace);
      dbf.setCoalescing(putCDATAIntoText);
      dbf.setExpandEntityReferences(!createEntityRefs);

      DocumentBuilder db=null;
      try {
         db = dbf.newDocumentBuilder();
      } catch (ParserConfigurationException pce) {
         throw new XMLParseException(pce);
      }

      try {
         OutputStreamWriter errorWriter = new OutputStreamWriter(System.err, outputEncoding);
         db.setErrorHandler(new MyErrorHandler(new PrintWriter(errorWriter, true)));
      } catch (UnsupportedEncodingException UEe) {
         throw new XMLParseException(UEe);
      }

      doc = null;
      try {
         doc = db.parse(new File(filename));
      } catch (SAXException se) {
         throw new XMLParseException(se.getMessage());
      } catch (IOException ioe) {
         throw new XMLParseException(ioe);
      }
   }

   public NodeList getTagList(String tagname) {
      return doc.getElementsByTagName(tagname);
   }

   public NodeList getTagListFromNode(Node x, String tagname) {
      return ((Element)x).getElementsByTagName(tagname);
   }

   public Map getNodeList(String nodeName) {
      Map o = new HashMap();
      NodeList nl = getTagList(nodeName);
      for (int i=0; i<nl.getLength(); i++) {
         Node x = nl.item(i);
         o.put(((Element)x).getAttribute("name"),((Element)x).getAttribute("class"));
      }
      return o;
   }

   public Node getNodeWithName(String nodeName, String nodeAttr, String nodeAttrVal) {
      NodeList nl = doc.getElementsByTagName(nodeName);
      for (int i=0; i<nl.getLength(); i++) {
         Node x = nl.item(i);
         if (((Element)x).getAttribute(nodeAttr).equals(nodeAttrVal)) return x;
      }
      return null;
   }

   public String lookup(String plugin, String parameter) {
      Node x = getNodeWithName("plugin","name",plugin);
      Node y = select(x,"property",parameter);
      if (null == y) return null;
      return (y.getFirstChild().getNodeValue());
   }

   private Node select(Node n, String element, String pl) {
      int type=n.getNodeType();
      Node x;
      switch (type) {
         case Node.ATTRIBUTE_NODE:
            break;
         case Node.CDATA_SECTION_NODE:
            break;
         case Node.COMMENT_NODE:
            break;
         case Node.DOCUMENT_FRAGMENT_NODE:
            break;
         case Node.DOCUMENT_NODE:
            break;
         case Node.DOCUMENT_TYPE_NODE:
            x = n.getFirstChild();
            while (x!=null && !x.getNodeName().equals(element)) x=x.getNextSibling();
            return x;
         case Node.ELEMENT_NODE:
            if (n.getNodeName().equals(element)) return n;
            x = null;
            for (x = n.getFirstChild(); x != null; x=x.getNextSibling()) {
               if (x!=null && x.getNodeName().equals(element) && ((Element)x).getAttribute("name").equals(pl)) break;
            }
            return x;
         case Node.ENTITY_NODE:
            break;
         case Node.ENTITY_REFERENCE_NODE:
            break;
         case Node.NOTATION_NODE:
            break;
         case Node.PROCESSING_INSTRUCTION_NODE:
            break;
         case Node.TEXT_NODE:
            break;
         default:
            System.out.println("UNSUPPORTED NODE: "+type);
            break;
      }
      return null;
   }

   public Document getDocument() {
      return doc;
   }

   public void replaceNode(Node oldNode, Node newNode) {
      doc.replaceChild(oldNode, newNode);
   }

   public void output(String filename) throws IOException {
      PrintWriter out = new PrintWriter(new FileOutputStream(new File(filename)));
      out.println(doc.toString());
   }

   // Error handler to report errors and warnings
   private static class MyErrorHandler implements ErrorHandler {
      /** Error handler output goes here */
      private PrintWriter out;

      MyErrorHandler(PrintWriter out) {
         this.out = out;
      }

      /**
       * Returns a string describing parse exception details
       */
      private String getParseExceptionInfo(SAXParseException spe) {
         String systemId = spe.getSystemId();
         if (systemId == null) {
            systemId = "null";
         }
         String info = "URI=" + systemId +
            " Line=" + spe.getLineNumber() +
            ": " + spe.getMessage();
         return info;
      }

      // The following methods are standard SAX ErrorHandler methods.
      // See SAX documentation for more info.

      public void warning(SAXParseException spe) throws SAXException {
         out.println("Warning: " + getParseExceptionInfo(spe));
      }

      public void error(SAXParseException spe) throws SAXException {
         String message = "Error: " + getParseExceptionInfo(spe);
         throw new SAXException(message);
      }

      public void fatalError(SAXParseException spe) throws SAXException {
         String message = "Fatal Error: " + getParseExceptionInfo(spe);
         throw new SAXException(message);
      }
   }
}
